import rox
from rox import g
import gobject
import os, sys, signal
from popen2 import Popen4
from rox.options import Option

o_device = Option("device","0,0,0")
o_overburn = Option("overburn",False)
o_speed = Option("speed",8)
o_dummy = Option("dummy",False)
o_blank = Option("blank","fast")

class Burner(rox.Window):
	def __init__(self,filename,blank=None):
		rox.Window.__init__(self)
		self.line=''
		self.aborting=False
		self.l=g.TextView()
		self.l.set_editable(False)
		self.l.set_wrap_mode(g.WRAP_WORD)
		self.progress = g.ProgressBar()
		self.progress.set_orientation(g.PROGRESS_LEFT_TO_RIGHT)
		swin = g.ScrolledWindow()
		swin.set_policy(g.POLICY_AUTOMATIC, g.POLICY_AUTOMATIC)
		swin.set_shadow_type(g.SHADOW_IN)
		swin.add(self.l)
#		swin.set_border_width(4)

		self.abort=rox.ButtonMixed(g.STOCK_NO,'Abort')
		self.abort.connect('clicked',self.abort_click)
		self.close=rox.ButtonMixed(g.STOCK_CLOSE,'Close')
		self.close.connect('clicked',self.close_click)
		self.close.set_sensitive(False)
		
		box2 = g.HBox(spacing=4)
#		box2.set_border_width(4)
		box2.pack_start(self.progress)
		box2.pack_end(self.close,False)
		box2.pack_end(self.abort,False)

		box = g.VBox(spacing=4)
		box.pack_start(swin)
		box.pack_end(box2,False)
		
		self.add(box)
		self.set_default_size(400,200)
		self.set_border_width(4)
		self.connect("delete-event",self.close_handler)
		if filename:
			t='Burning '+os.path.basename(filename)+' to CD'
		elif blank:
			t='Blanking CD-RW'
#			t='Burning unsaved ISO to CD'
#		if simulate.int_value: t='Fake'+t
		self.set_title(t)

		self.watch=None
	
		cmd='cdrecord -v gracetime=2 '
		if blank: cmd+='blank='+o_blank.value+' '
		cmd+='dev='+o_device.value+' '
		if o_dummy.int_value: cmd+='-dummy '
		cmd+='speed='+str(o_speed.int_value)+' '
		if filename:
			if o_overburn.int_value: cmd+='-overburn '
			cmd+='-data "'+filename+'"'
		
#		rox.info(cmd)
#		return

		self.l.get_buffer().connect('changed',self.auto_scroll)
		
#		os.chdir(os.path.dirname(filename)) #not needed since tmp-tocfile have full pathnames..
		self.burner=Popen4(cmd)
		self.fo=self.burner.fromchild

		self.watch = gobject.io_add_watch(self.fo,gobject.IO_IN | gobject.IO_HUP | gobject.IO_PRI,self.logwatch)
		
		self.show_all()
		
	def auto_scroll(self,buf):
		self.l.scroll_mark_onscreen(buf.get_insert())
		
	def logwatch(self,fo,x):
		buf=self.l.get_buffer()
		c = fo.read(1)
		
		if not c:
			gobject.source_remove(self.watch)

		if self.burner.poll() > -1:
			self.close.set_sensitive(True)
			self.abort.set_sensitive(False)
			
		if self.burner.poll() > 0:
			self.progress.set_text('Burnprocess aborted')
			self.progress.set_fraction(1)
#			return True
	
		if self.burner.poll() == 0:
			self.progress.set_text('Burn finished successfully')
			self.progress.set_fraction(1)
		
		if c == '\n' or c == '\r':
			line = self.line
			self.line = ''
			if not line.find("seconds") > -1:
				buf.insert(buf.get_end_iter(),line+'\n')
			if not self.aborting:

				for s in [ 'Starting new', 'Fixating' ]:
					if line.find(s) > -1:
						self.progress.set_text(line)
				if line.find('BLANK mode') > -1:
					self.progress.set_text('Erasing CD, please wait...')
					self.progress.set_fraction(0.5)
					
			if line.find('written') > -1 and line.find('of') > -1:
				fields=line.split()
				x=float(fields[2])
				y=float(fields[4])
				if y>0: f=x/y
				else: f=1
				self.progress.set_fraction(f)
				self.progress.set_text(line[:line.find('(')])
		else:
			self.line+=c
		

		return True
		
	def kill_burner(self):
		self.aborting=True
		self.progress.set_text('Waiting for burnprocess to terminate...')
		os.kill(self.burner.pid,signal.SIGQUIT)
#		self.abort.set_sensitive(False)

	def abort_click(self,button):
		if self.burner.poll() == -1:
			self.kill_burner()
			
	def close_click(self,button):
		self.close_handler(None)
		
	def close_handler(self,wid,ev=None):
		if self.burner.poll() == -1:
			self.kill_burner()
			return True
		gobject.source_remove(self.watch)
		self.fo.close()
#		self.tmpfile.close()
		if not ev: self.destroy()
		return False
