import rox
from rox import g
import gobject
import os, sys, signal
from popen2 import Popen4
from rox.options import Option

o_rockridge = Option("rock_ridge",True)
o_joliet = Option("joliet",True)
o_followlinks = Option("follow_links",False)

class MkImage(rox.Window):
	def __init__(self,liststore,fn,cb):
		rox.Window.__init__(self)

#############################################
		cmd='mkisofs'
		if o_rockridge.int_value: cmd+=' -r'
		if o_joliet.int_value: cmd+=' -J'
		if o_followlinks.int_value: cmd+=' -f'
		cmd+=' -graft-points -o %s ' % (fn)
		for x in liststore:
			cmd+=x[1]+'='+x[2]+' '

		title="Creating ISO Image"
		self.cmdname="mkisofs"

#############################################

		self.done_cb=cb
		self.filename=fn
		self.aborting=False
		self.l=g.TextView()
		self.l.set_editable(False)
		self.l.set_wrap_mode(g.WRAP_WORD)
		self.progress = g.ProgressBar()
		self.progress.set_orientation(g.PROGRESS_LEFT_TO_RIGHT)
		swin = g.ScrolledWindow()
		swin.set_policy(g.POLICY_AUTOMATIC, g.POLICY_AUTOMATIC)
		swin.set_shadow_type(g.SHADOW_IN)
		swin.add(self.l)

		self.abort=rox.ButtonMixed(g.STOCK_NO,'Abort')
		self.abort.connect('clicked',self.abort_click)
		self.close=rox.ButtonMixed(g.STOCK_CLOSE,'Close')
		self.close.connect('clicked',self.close_click)
		self.close.set_sensitive(False)
		
		box2 = g.HBox(spacing=4)
		box2.pack_start(self.progress)
		box2.pack_end(self.close,False)
		box2.pack_end(self.abort,False)

		box = g.VBox(spacing=4)
		box.pack_start(swin)
		box.pack_end(box2,False)
		
		self.add(box)
		self.set_default_size(400,200)
		self.set_border_width(4)
		self.connect("delete-event",self.close_handler)
		self.set_title(title)

		self.l.get_buffer().connect('changed',self.auto_scroll)

		self.burner=Popen4(cmd)
		fo=self.burner.fromchild
		self.watch = gobject.io_add_watch(fo,gobject.IO_IN | gobject.IO_HUP | gobject.IO_PRI,self.logwatch)
		gobject.timeout_add(250,self.poller)
		g.gdk.flush()
		self.show_all()
		
	def auto_scroll(self,buf):
		self.l.scroll_mark_onscreen(buf.get_insert())
		
	def add_line(self,line):
		buf=self.l.get_buffer()
		buf.insert(buf.get_end_iter(),line+'\n')
		
	def poller(self):
		poll = self.burner.poll()
		if poll > -1:
			self.progress.set_fraction(1)
			self.retcode = poll
			
		if poll > 0:
			self.progress.set_text(self.cmdname+' aborted')
			return False
		elif poll == 0:
			self.progress.set_text(self.cmdname+' finished successfully')
			self.success=True
			return False

		self.progress.pulse()			
		return True
		
	def logwatch(self,fo,x):
		l = fo.readline()
		if not l:
			gobject.source_remove(self.watch)
			self.close.set_sensitive(True)
			self.abort.set_sensitive(False)
		else:
			self.add_line(l.strip())
		return True
				
	def kill_burner(self):
		self.aborting=True
		self.add_line('Waiting for '+self.cmdname+' to terminate...')
		os.kill(self.burner.pid,signal.SIGQUIT)

	def abort_click(self,button):
		if self.burner.poll() == -1:
			self.kill_burner()
			
	def close_click(self,button):
		self.close_handler(None)
		
	def close_handler(self,wid,ev=None):
		if self.burner.poll() == -1:
			self.kill_burner()
			return True
		gobject.source_remove(self.watch)
		if not ev:
			self.done_cb(self.filename,self.retcode)
			self.destroy()
		return False
